package cn.wise.sc.cement.business.controller;


import cn.wise.sc.cement.business.entity.SysCoupon;
import cn.wise.sc.cement.business.mapper.SysCouponMapper;
import cn.wise.sc.cement.business.model.PageTableRequest;
import cn.wise.sc.cement.business.service.ISysCouponService;
import cn.wise.sc.cement.business.wrapper.WrapMapper;
import cn.wise.sc.cement.business.wrapper.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDateTime;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author ztw
 * @since 2020-06-17
 */
@Api(tags = "优惠券管理")
@RestController
@RequestMapping("/business/sys-coupon")
public class SysCouponController {
	private static final Logger log = LoggerFactory.getLogger("SysCouponController");

	@Resource
	private SysCouponMapper couponMapper;
	@Autowired
	private ISysCouponService couponService;

	@ApiOperation(value = "优惠券列表(Wrapper实现)")
	@ApiImplicitParams(value = {
			@ApiImplicitParam(name = "current", value = "当前页数(从1开始)", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "size", value = "当前每页显示数", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "couponName", value = "企业名称", paramType = "query", dataType = "String"),
			@ApiImplicitParam(name = "status", value = "状态：0禁用, 1启用", paramType = "query", dataType = "Integer")
	})
	@GetMapping("getList")
	public Wrapper getList(PageTableRequest request, String couponName, Integer status) {
		try {
			QueryWrapper<SysCoupon> wrapper = new QueryWrapper<>();
			if (StringUtils.isNotBlank(couponName)) {
				wrapper.like("coupon_name", couponName);
			}
			if (status != null) {
				wrapper.eq("status", status);
			}
			wrapper.orderByDesc("create_time");
			IPage<SysCoupon> page = new Page<>(request.getCurrent(), request.getSize());
			page = couponMapper.selectPage(page, wrapper);
			return WrapMapper.wrap(200, "成功", page);
		} catch (Exception e) {
			log.debug("优惠券列表{}", e);
		}
		return WrapMapper.error("失败");

	}

	@ApiOperation(value = "添加优惠券")
	@PostMapping("save")
	public Wrapper save(@RequestBody SysCoupon coupon){
		try {
			coupon.setStatus(1);
			coupon.setIsDelete(1);
			coupon.setCreateTime(LocalDateTime.now());
			couponService.save(coupon);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("添加优惠券{}",e);
		}
		return WrapMapper.error("失败");
	}


	@ApiOperation(value = "修改优惠券")
	@PostMapping("update")
	public Wrapper update(@RequestBody SysCoupon coupon){
		try {
			SysCoupon c = couponService.getById(coupon.getId());
			if(c == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			couponService.updateById(coupon);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("修改优惠券{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "优惠券详情")
	@GetMapping("{id}")
	public Wrapper getById(@PathVariable Long id){
		try {
			SysCoupon c = couponService.getById(id);
			if(c == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			return WrapMapper.wrap(200,"成功",c);
		}catch (Exception e){
			log.debug("优惠券详情{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "删除优惠券")
	@DeleteMapping("/{id}")
	public Wrapper delete(@PathVariable Long id) {
		try {
			SysCoupon c = couponService.getById(id);
			if (c == null ) {
				return WrapMapper.wrap(400,"优惠券不存在");
			}
			couponService.removeById(id);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("删除优惠券{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "领取优惠券")
	@DeleteMapping("/receive/{id}")
	public Wrapper receive(@PathVariable Long id) {
		try {
			SysCoupon c = couponService.getById(id);
			if (c == null ) {
				return WrapMapper.wrap(400,"优惠券不存在");
			}
			c.setIsReceive(1);
			couponService.updateById(c);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("领取优惠券{}",e);
		}
		return WrapMapper.error("失败");
	}







}

