package cn.wise.sc.cement.tcdri.service.impl;

import cn.hutool.core.util.StrUtil;
import cn.wise.sc.cement.tcdri.entity.Client;
import cn.wise.sc.cement.tcdri.entity.Project;
import cn.wise.sc.cement.tcdri.exception.BusinessExceptionEnum;
import cn.wise.sc.cement.tcdri.mapper.ClientMapper;
import cn.wise.sc.cement.tcdri.model.BaseResponse;
import cn.wise.sc.cement.tcdri.model.PageQuery;
import cn.wise.sc.cement.tcdri.model.query.ClientQuery;
import cn.wise.sc.cement.tcdri.model.vo.ClientVo;
import cn.wise.sc.cement.tcdri.service.IClientService;
import cn.wise.sc.cement.tcdri.service.IProjectService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author ztw
 * @since 2020-08-07
 */
@Service
public class ClientServiceImpl extends ServiceImpl<ClientMapper, Client> implements IClientService {

    final
    IProjectService iProjectService;

    public ClientServiceImpl(IProjectService iProjectService) {
        this.iProjectService = iProjectService;
    }

    @Override
    public BaseResponse<IPage<ClientVo>> getPage(PageQuery pageQuery, Integer status, String region, String principal) {

        QueryWrapper<Client> qw = new QueryWrapper<>();
        qw.orderByDesc("id");
        if (status != null) {
            qw.eq("status", status);
        }
        if (StrUtil.isNotBlank(region)) {
            qw.like("region", region);
        }
        if (StrUtil.isNotBlank(principal)) {
            qw.like("principal", principal);
        }
        IPage<Client> page = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        this.page(page);
        IPage<ClientVo> rts = new Page<>(pageQuery.getPageNo(), pageQuery.getPageSize());
        //关联项目名
        List<Client> records = page.getRecords();
        if (records == null) {
            return BaseResponse.okData(rts);
        }
        List<Integer> clientIds = records.stream().map(Client::getId).collect(Collectors.toList());
        BusinessExceptionEnum.COLLECTION_NOT_ILLEGAL.assertCollectionNotILLEGAL(clientIds);
        QueryWrapper<Project> qwProject = new QueryWrapper<>();
        qwProject.in("client_id",clientIds);
        List<Project> projects = iProjectService.list(qwProject);

        List<ClientVo> rtsRe = new ArrayList<>(page.getRecords().size());
        for (Client client : page.getRecords()) {
            ClientVo clientVo = client.toVo();
            List<String> projectNames = new ArrayList<>();
            projects.forEach(arg->{
                if (arg.getClientId().intValue() == client.getId()){
                    projectNames.add(arg.getName());
                }
            });
            clientVo.setProjects(projectNames);
            rtsRe.add(clientVo);
        }
        rts.setRecords(rtsRe);
        return BaseResponse.okData(rts);
    }

    @Override
    public BaseResponse<List<ClientVo>> getList() {
        QueryWrapper<Client> qw = new QueryWrapper<>();
        qw.eq("status", 1);
        List<Client> list = this.list(qw);
        List<ClientVo> rts = new ArrayList<>(list.size());
        list.forEach(arg -> {
            ClientVo clientVo = new ClientVo();
            clientVo.setId(arg.getId());
            clientVo.setName(arg.getName());
            rts.add(clientVo);
        });
        return BaseResponse.okData(rts);
    }

    @Override
    public BaseResponse<ClientVo> create(final ClientQuery clientQuery) {
        //判断名字是否重复
        if (clientIsExisting(clientQuery.getName())) {
            return BaseResponse.errorMsg("当前单位名已存在!");
        }
        Client crate = Client.crate(clientQuery);
        this.save(crate);
        return BaseResponse.okData(crate.toVo());
    }

    @Override
    public BaseResponse<ClientVo> update(final ClientQuery clientQuery) {
        //判断名字是否重复
        if (clientIsExisting(clientQuery.getName())) {
            return BaseResponse.errorMsg("当前单位名已存在!");
        }
        Client update = Client.crate(clientQuery);
        update.setId(clientQuery.getId());
        this.updateById(update);
        return BaseResponse.okData(update.toVo());
    }

    @Override
    public BaseResponse<Boolean> status(Integer status, Integer id) {
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(status);
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(id);

        Client update = new Client();
        update.setId(id);
        update.setStatus(status == 1);

        return BaseResponse.okData(this.updateById(update));
    }


    /**
     * 判断单位是否重复
     *
     * @param name 单位名
     * @return bool
     */
    private boolean clientIsExisting(String name) {
        BusinessExceptionEnum.BAD_LICENCE_TYPE.assertNotNull(name);
        //判断名字是否重复
        QueryWrapper<Client> qw = new QueryWrapper<>();
        qw.eq("name", name);
        qw.eq("status", 1);
        int count = this.count(qw);
        return count > 0;
    }
}
